module moderator

use definitions
use math

contains

!================================================================!
!= setmoderator define the physical shape of the moderator	=!
!= The moderator is box shaped and starts in (0,0,0)		=!
!= and extends to (xmax,ymax,zmax) which are set here.		=!
!= A sphere deformation may be used at the output end:		=!
!= described by a center r0_sphere and radius r_sphere		=!
!= This sphere should generaly center with z>zmax		=!
!= The sphere describes a region without moderator		=!
!= The program works in units of mm (milimeters)		=!
!= If cylinder=0 the moderator resembels a small block.		=!
!= If cylinder=1 the moderator approximates the main moderator  =!
!=  			( which does not support voids )	=!
!= If cylinder_v=1 the small moderator has a cylinder void	=!
!=								=!
!= Other voids can be added but it will need a new intervoid	=!
!=  routine for that geometry. The geometry is required to be	=!
!=  concave, otherwise the program will not take care of 	=!
!=  certain classes of special cases.				=!
!================================================================!

subroutine setmoderator
implicit none

if ( cylinder==1) then			! Main moderator
 xmax=410
 ymax=410
 zmax=310
else

 xmax=220. 		
 ymax=220.		
 zmax=68.		

 if (flat==1) then			! Flat moderator
   r0_sphere(1)=xmax/2.
   r0_sphere(2)=ymax/2.
   r0_sphere(3)=zmax+100
   r_sphere=1
   safezmax=r0_sphere(3)-r_sphere  

   r_cylinder=1
   h_cylinder=1
   ccentrum(1)=xmax*0.5
   ccentrum(2)=ymax*0.5
   ccentrum(3)=zmax+100

   czmax=ccentrum(3)+0.5*h_cylinder
   czmin=ccentrum(3)-0.5*h_cylinder
   safezmax=ccentrum(3)-0.5*h_cylinder

 else

  if (cylinder_v==0) then 		! Sphere void
   r0_sphere(1)=xmax/2.+1.5
   r0_sphere(2)=ymax/2.-0.85
   r0_sphere(3)=zmax+25
   r_sphere=sqrt(25**2+50.105**2)
   safezmax=r0_sphere(3)-r_sphere

   ccentrum(:)=0
   ccentrum(3)=zmax+100
   r_cylinder=1
   h_cylinder=1

  else if (cylinder_v==1) then		! Cylinder void

   r_cylinder=10
   h_cylinder=40
   ccentrum(1)=xmax*0.5
   ccentrum(2)=ymax*0.5
   ccentrum(3)=zmax-0.5*h_cylinder

   czmax=ccentrum(3)+0.5*h_cylinder
   czmin=ccentrum(3)-0.5*h_cylinder
   safezmax=ccentrum(3)-0.5*h_cylinder

   r0_sphere(:)=0
   r0_sphere(3)=zmax+100
   r_sphere=1 
 
  endif
 endif
endif

end subroutine setmoderator

!================================================================!
!= posmod checks if the position is in the moderator.		=!
!= outputs integer posstatus					=!
!= posstatus = 1 if r is in safe simple box			=!
!= posstatus = 2 if r is in care zone and out of sphere		=!
!= posstatus = 3 if r is in care zone and in void		=!
!= posstatus = 4 if r is out of simple box			=!
!=								=!
!= The simple box is defined as the box without the void.	=!
!=								=!
!= This routine has been checked with sample points, and no	=!
!=	   errors were found					=!
!================================================================!

subroutine posmod
implicit none

if (r(1).le.xmax.and.r(2).le.ymax.and.r(3).le.zmax &
    .and.r(1).ge.0..and.r(2).ge.0..and.r(3).ge.0.) then

	if (r(3).gt.safezmax) then
           if (cylinder_v==0) then
	     if (distance(r,r0_sphere).lt.r_sphere) then
		posstatus=3
	     else
		posstatus=2
	     endif
	   else if (cylinder_v==1) then
	     if ((r(1)-ccentrum(1))**2+(r(2)-ccentrum(2))**2.lt.r_cylinder**2) then
		posstatus=3
	     else
		posstatus=2
	     endif
	   endif
	else
	   posstatus=1
	endif

else
   posstatus=4

endif

end subroutine posmod

!================================================================!
!= concavecheck picks three points between r and r_old, and	=!
!=   checks if any of those are in the sphere void.		=!
!= Output: ccheck=1 if one or more points is in the sphere	=!
!= Output: ccheck=0 if no points is in the sphere		=!
!================================================================!

subroutine concavecheck
implicit none
real,dimension(3)	:: point
real			:: sep
integer			:: j

ccheck=0

sep=distance(r(:),r_old(:))

if (cylinder_v==0) then
 if (distance(r_temp(:),r0_sphere(:)).gt.r_sphere) then
	do j=1,3
	 point(:)=r_old(:)+0.25*real(j)*sep*r_direc(:)
	 if (distance(point,r0_sphere).lt.r_sphere) then
	 ccheck=1
	 endif
	enddo
 endif
else if (cylinder_v==1) then
 if ((r_temp(1)-ccentrum(1))**2+(r_temp(2)-ccentrum(2))**2.lt.r_cylinder**2 &
	.and.r_temp(3).lt.czmax.and.r_temp(3).gt.czmin) then
	ccheck=0
 else
	do j=1,3
	 point(:)=r_old(:)+0.25*real(j)*sep*r_direc(:)
	 if ((point(1)-ccentrum(1))**2+(point(2)-ccentrum(2))**2.lt.r_cylinder**2 &
	 .and.point(3).gt.czmin.and.point(3).lt.czmax) then
	 ccheck=1
	 endif
	enddo
 endif
endif

end subroutine concavecheck

!================================================================!
!= SAFERUN ONLY USED FOR DEBUGGING				=!
!= saferun avoids loops at the cost of possible statistical 	=!
!=  error. If both r and r_old has suspicious coordinates,	=!
!=  the neutron will be moved quite far into the moderator	=!
!=  to aviod corner loops					=!
!= Saferun should be rewritten to do much more subtle changes	=!
!=  if it is wanted for very large datasets. It is not used 	=!
!=  anymore since the program does not return errors. 		=!
!= We suggest just dropping neutrons with errors if the program	=!
!=  is used for very large datasets and errors not found by us 	=!
!=  is present because of very low probabilities. 		=!
!================================================================!

subroutine saferun

!if (r_old(1).gt.xmax-0.001.or.r_old(2).gt.ymax-0.001.or.r_old(3).gt.zmax-0.001 &
!    .or.r_old(1).lt.0.001.or.r_old(2).lt.0.001.or.r_old(3).lt.0.001) then

if (r_old(1).gt.xmax.or.r_old(2).gt.ymax.or.r_old(3).gt.zmax &
    .or.r_old(1).lt.0..or.r_old(2).lt.0..or.r_old(3).lt.0.) then

  if (r(1).gt.xmax-0.001.or.r(2).gt.ymax-0.001.or.r(3).gt.zmax-0.001 &
    .or.r(1).lt.0.001.or.r(2).lt.0.001.or.r(3).lt.0.001) then

	if (r(1).ge.xmax-0.0001) r(1)=xmax-0.001
	if (r(1).le.0.0001) 	 r(1)=0.001
	if (r(2).ge.ymax-0.0001) r(2)=ymax-0.001
	if (r(2).le.0.0001) 	 r(2)=0.001
	if (r(3).ge.zmax-0.0001) r(3)=zmax-0.001
	if (r(3).le.0.0001) 	 r(3)=0.001

	if (error.gt.300) stuck=1

	error=error+1
	terror=terror+1
  end if

end if

end subroutine saferun

end module moderator